﻿using System.Windows;
using HalconDotNet;

namespace CountBeansAndPeas
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public HTuple Index = new HTuple(0);

        public MainWindow()
        {
            InitializeComponent();
        }

        private void HSmartWindowControlWPF_Loaded(object sender, RoutedEventArgs e)
        {
            HSmartWindowControlWPF.HalconWindow.SetDraw("margin");
            HSmartWindowControlWPF.HalconWindow.SetLineWidth(3);
            HOperatorSet.SetSystem("store_empty_region", "false");
            HSmartWindowControlWPF.HalconWindow.SetFont("Consolas-Bold-16");
        }

        private void CountBeansAndPeasButton_Click(object sender, RoutedEventArgs e)
        {
            // Read image and apply threshold
            HImage Image = new HImage();
            Image.ReadImage("images_beans_and_peas/beans_and_peas_0" + new HTuple (Index % 4 + 1));
            HRegion Region = Image.Threshold(30.0, 255.0);
            // Seperate beans from peas
            HRegion RegionErosion = Region.ErosionCircle(31.5);
            HRegion ConnectedRegions = RegionErosion.Connection();
            HRegion RegionDilation = ConnectedRegions.DilationCircle(31.5);
            HRegion Peas = RegionDilation.SelectShape("roundness", "and", 0.85, 1.0);
            HRegion Beans = RegionDilation.Difference(Peas);
            // Count beans and peas
            HTuple NumberOfBeans = new HTuple(Beans.CountObj());
            HTuple NumberOfPeas = new HTuple(Peas.CountObj());
            // Visualization
            VisualizeBeansAndPeas(Image, Beans, Peas, NumberOfBeans, NumberOfPeas);
            Index++;
        }

        public void VisualizeBeansAndPeas(HImage Image, HRegion Beans, HRegion Peas, HTuple NumberOfBeans, HTuple NumberOfPeas)
        {
            HTuple Width, Height;
            HTuple RowBeans, ColumnBeans;
            HTuple RowPeas, ColumnPeas;
            HXLDCont CrossBeans = new HXLDCont();
            HXLDCont CrossPeas = new HXLDCont();
            //Display image, beans, and peas
            Image.GetImageSize(out Width, out Height);
            HSmartWindowControlWPF.HalconWindow.SetPart(0, 0, (int)Height - 1, (int)Width - 1);
            HSmartWindowControlWPF.HalconWindow.DispObj(Image);
            //Display beans
            HSmartWindowControlWPF.HalconWindow.SetColor("#23a0e9");
            HSmartWindowControlWPF.HalconWindow.DispObj(Beans);
            HTuple AreaBeans = Beans.AreaCenter(out RowBeans, out ColumnBeans);
            CrossBeans.GenCrossContourXld(RowBeans, ColumnBeans, (HTuple)20.0, 0);
            HSmartWindowControlWPF.HalconWindow.DispObj(CrossBeans);
            // Display peas
            HSmartWindowControlWPF.HalconWindow.SetColor("#fbba00");
            HSmartWindowControlWPF.HalconWindow.DispObj(Peas);
            HTuple AreaPeas = Peas.AreaCenter(out RowPeas, out ColumnPeas);
            CrossPeas.GenCrossContourXld(RowPeas, ColumnPeas, (HTuple)20.0, 0);
            HSmartWindowControlWPF.HalconWindow.DispObj(CrossPeas);
            //Display text (number of beans and peas)
            HSmartWindowControlWPF.HalconWindow.DispText("Image " + (Index % 4 + 1) + "/4"+ "\nNumber of beans: " + NumberOfBeans + "\nNumber of peas: " + NumberOfPeas, "window", "top", "left", "black", "box_color", "#ffffffbb");
            return;
        }
    }
}